#
# DecoMecha - diving decompression library.
#
# Copyright (C) 2013 by Artur Wroblewski <wrobell@pld-linux.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

"""
Better gas sanity unit tests.
"""

from decomecha import DecoEngine

from .util import TestCase


class AirEANGasTestCase(TestCase):
    """
    Better gas sanity unit tests.
    """
    def test_air_ean50(self):
        """
        Test better gas: Air vs. Air+EAN50
        """
        d1 = DecoEngine()
        d1.add_segment(45, 25)
        d1.add_gas(0, 21)

        i1 = d1.calculate()

        d2 = DecoEngine()
        d2.add_segment(45, 25)
        d2.add_gas(0, 21)
        d2.add_gas(22, 50)

        i2 = d2.calculate()

        self._verify(i1, i2, 15)


    def test_air_ean50_o2(self):
        """
        Test better gas: Air+EAN50 vs. Air+EAN50+O2
        """
        d1 = DecoEngine()
        d1.add_segment(45, 25)
        d1.add_gas(0, 21)
        d1.add_gas(22, 50)

        i1 = d1.calculate()

        d2 = DecoEngine()
        d2.add_segment(45, 25)
        d2.add_gas(0, 21)
        d2.add_gas(22, 50)
        d2.add_gas(6, 100)

        i2 = d2.calculate()

        self._verify(i1, i2, 6)


    def test_air_ean27(self):
        """
        Test better gas: Air vs. EAN27
        """
        d1 = DecoEngine()
        d1.add_segment(40, 25)
        d1.add_gas(0, 21)

        i1 = d1.calculate()

        d2 = DecoEngine()
        d2.add_segment(40, 25)
        d2.add_gas(0, 27)

        i2 = d2.calculate()

        self._verify(i1, i2, 12)


    def test_air_ean32_ndl(self):
        """
        Test better gas: Air vs. EAN32 (NDL)
        """
        d1 = DecoEngine()
        d1.add_segment(30, 10)
        d1.add_gas(0, 21)

        i1 = d1.calculate()

        d2 = DecoEngine()
        d2.add_segment(30, 10)
        d2.add_gas(0, 32)

        i2 = d2.calculate()

        # Buhlmann tables, the diff for Air and EAN32 is 8 min
        self.assertTrue(i2.ndl - i1.ndl > 5, '{} vs. {}'.format(i1.ndl, i2.ndl))


    def test_air_tx_21_55(self):
        """
        Test better gas: TX 21/55 vs. Air
        """
        d1 = DecoEngine()
        d1.add_segment(45, 30)
        d1.add_gas(0, 21, 55)

        i1 = d1.calculate()

        d2 = DecoEngine()
        d2.add_segment(45, 30)
        d2.add_gas(0, 21)

        i2 = d2.calculate()

        self._verify(i1, i2, 15)



class CCRBetterGasTestCase(TestCase):
    """
    Better gas sanity unit tests for CCR dives.
    """
    def test_oc_air(self):
        """
        Test better gas: OC vs. CCR on air
        """
        d1 = DecoEngine()
        d1.add_segment(45, 25)
        d1.add_gas(0, 21)
        i1 = d1.calculate()

        d2 = DecoEngine()
        d2.add_segment(45, 25)
        d2.add_gas(0, 21)
        d2.add_setpoint(0, 1.3)
        i2 = d2.calculate()

        self._verify(i1, i2, better_at=15)


    def test_oc_tx(self):
        """
        Test better gas: OC vs. CCR on TX 14/55
        """
        d1 = DecoEngine()
        d1.add_segment(60, 22)
        d1.add_gas(0, 14, 55)
        i1 = d1.calculate()

        d2 = DecoEngine()
        d2.add_segment(60, 22)
        d2.add_gas(0, 14, 55)
        d2.add_setpoint(0, 1.3)
        i2 = d2.calculate()

        self._verify(i1, i2, better_at=27)


    def test_oc_ndl(self):
        """
        Test better gas: NDL time of OC and CCR dives
        """
        d1 = DecoEngine()
        d1.add_segment(20, 20)
        d1.add_gas(0, 21)
        i1 = d1.calculate()

        d2 = DecoEngine()
        d2.add_segment(20, 20)
        d2.add_gas(0, 21)
        d2.add_setpoint(0, 1.3)
        i2 = d2.calculate()

        # avoid deco and non-zero NDL for this test
        assert i1.ndl > 0
        assert i2.ndl > 0
        self.assertTrue(i1.ndl < i2.ndl, '{} vs. {}'.format(i1.ndl, i2.ndl))


    def test_better_setpoint(self):
        """
        Test better gas: CCR with better setpoint
        """
        d1 = DecoEngine()
        d1.add_segment(60, 22)
        d1.add_gas(0, 14, 55)
        d1.add_setpoint(0, 1.1)
        i1 = d1.calculate()

        d2 = DecoEngine()
        d2.add_segment(60, 22)
        d2.add_gas(0, 14, 55)
        d2.add_setpoint(0, 1.3)
        i2 = d2.calculate()

        self._verify(i1, i2, better_at=12)


# vim: sw=4:et:ai
